"""
This type stub file was generated by pyright.
"""

def authenticate_message(
    msg: bytes,
    authserv_id: str,
    *,
    dkim: bool = ...,
    dmarc: bool = ...,
    arc: bool = ...,
    spf: bool = ...,
) -> str: ...
